@@ -19,9 +19,7 @@
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-   02111-1307, USA.
+   along with this program; if not, see <http://www.gnu.org/licenses/>.
 
    The GNU General Public License is contained in the file COPYING.
 */
@@ -373,7 +371,8 @@ Bool get_debug_pos(BBCC* bbcc, Addr addr, AddrPos* p)
 	found_file_line = debug_cache_info[cachepos];
     }
     else {
-	found_file_line = VG_(get_filename_linenum)(addr,
+        DiEpoch ep = VG_(current_DiEpoch)();
+	found_file_line = VG_(get_filename_linenum)(ep, addr,
 						    &file,
 						    &dir,
 						    &(p->line));
@@ -1302,7 +1301,27 @@ static VgFile *new_dumpfile(int tid, const HChar* trigger)
 		CLG_(clo).dump_bb    ? " bb" : "",
 		CLG_(clo).dump_line  ? " line" : "");
 
-   /* "events:" line */
+  /* Some (optional) "event:" lines, giving long names to events. */
+   switch (CLG_(clo).collect_systime) {
+     case systime_no: break;
+     case systime_msec:
+        VG_(fprintf)(fp, "event: sysTime : sysTime (elapsed ms)\n");
+        break;
+     case systime_usec:
+        VG_(fprintf)(fp, "event: sysTime : sysTime (elapsed us)\n");
+        break;
+     case systime_nsec:
+        VG_(fprintf)(fp, "event: sysTime : sysTime (elapsed ns)\n");
+        VG_(fprintf)(fp, "event: sysCpuTime : sysCpuTime (system cpu ns)\n");
+        break;
+     default:
+        tl_assert(0);
+   }
+
+   /* "events:" line
+      Note: callgrind_annotate expects the "events:" line to be the last line
+      of the PartData.  In other words, this line is before the first line
+      of the PartData body. */
    HChar *evmap = CLG_(eventmapping_as_string)(CLG_(dumpmap));
    VG_(fprintf)(fp, "events: %s\n", evmap);
    VG_(free)(evmap);
