@@ -145,11 +145,29 @@ SysRes VG_(mk_SysRes_ppc32_linux) ( UInt val, UInt cr0so ) {
    return res;
 }
 
-/* As per ppc32 version, cr0.so must be in l.s.b. of 2nd arg */
-SysRes VG_(mk_SysRes_ppc64_linux) ( ULong val, ULong cr0so ) {
+/* As per ppc32 version, for the sc instruction cr0.so must be in
+   l.s.b. of 2nd arg.
+   For the scv 0 instruction, the return value indicates failure if
+   it is -4095..-1 (i.e., it is >= -MAX_ERRNO (-4095) as an unsigned
+   comparison), in which case the error value is the negated return value. */
+SysRes VG_(mk_SysRes_ppc64_linux) ( ULong val, ULong cr0so, UInt flag ) {
    SysRes res;
-   res._isError = (cr0so & 1) != 0;
-   res._val     = val;
+
+   if (flag == SC_FLAG) {
+      /* sc instruction */
+      res._isError = (cr0so & 1) != 0;
+      res._val     = val;
+   } else if (flag == SCV_FLAG) {
+      /* scv instruction */
+      if ( (Long)val >= -4095 && (Long)val <= -1) {
+         res._isError = True;
+         res._val = (ULong)(-val);
+      } else {
+         res._isError = False;
+         res._val = (ULong)(val);
+      }
+   } else
+      vg_assert(0);
    return res;
 }
 
@@ -346,6 +364,42 @@ SysRes VG_(mk_SysRes_amd64_solaris) ( Bool isErr, ULong val, ULong val2 )
    return res;
 }
 
+
+#elif defined(VGO_freebsd)
+
+SysRes VG_(mk_SysRes_x86_freebsd) ( UInt val, UInt val2, Bool err ) {
+   SysRes r;
+   r._isError = err;
+   r._val = val;
+   r._val2 = val2;
+   return r;
+}
+
+SysRes VG_(mk_SysRes_amd64_freebsd) ( ULong val, ULong val2, Bool err ) {
+   SysRes r;
+   r._isError = err;
+   r._val = val;
+   r._val2 = val2;
+   return r;
+}
+
+/* Generic constructors. */
+SysRes VG_(mk_SysRes_Error) ( UWord err ) {
+   SysRes r;
+   r._val     = err;
+   r._val2    = 0;
+   r._isError = True;
+   return r;
+}
+
+SysRes VG_(mk_SysRes_Success) ( UWord res ) {
+   SysRes r;
+   r._val     = res;
+   r._val2    = 0;
+   r._isError = False;
+   return r;
+}
+
 #else
 #  error "Unknown OS"
 #endif
@@ -541,6 +595,12 @@ asm(
 "        addi         2,2,.TOC.-0b@l\n"
 "        .localentry do_syscall_WRK, .-do_syscall_WRK\n"
 "#endif"                            "\n"
+/* Check which system call instruction to issue*/
+"        ld   8, 56(3)\n"  /* arg 7 holds sc/scv flag */
+"        cmpdi 8,1\n"      /* check sc/scv flag not equal to SC_FLAG*/
+"        bne  issue_scv\n"
+
+/* setup and issue the sc instruction */
 "        std  3,-16(1)\n"  /* stash arg */
 "        ld   8, 48(3)\n"  /* sc arg 6 */
 "        ld   7, 40(3)\n"  /* sc arg 5 */
@@ -556,6 +616,38 @@ asm(
 "        srwi 3,3,28\n"
 "        andi. 3,3,1\n"
 "        std  3,8(5)\n"    /* argblock[1] = cr0.s0 & 1 */
+"        blr\n"            /* return */
+
+/*  setup to do scv instruction */
+"issue_scv: "
+/* The scv instruction requires a new stack frame */
+"        stdu    1,-80(1)\n"
+"        std     27,40(1)\n" /* save r27 to stack frame */
+"        mflr    27\n"       /* Get link register */
+"        std     27,16(1)\n" /* Save link register */
+
+/* setup and issue the scv instruction */
+"        std  3,-16(1)\n"  /* stash arg */
+"        ld   8, 48(3)\n"  /* sc arg 6 */
+"        ld   7, 40(3)\n"  /* sc arg 5 */
+"        ld   6, 32(3)\n"  /* sc arg 4 */
+"        ld   5, 24(3)\n"  /* sc arg 3 */
+"        ld   4, 16(3)\n"  /* sc arg 2 */
+"        ld   0,  0(3)\n"  /* sc number */
+"        ld   3,  8(3)\n"  /* sc arg 1 */
+
+"        .machine push\n"
+"        .machine \"power9\"\n"
+"        scv  0\n"
+"        .machine pop\n"
+"        ld   5,-16(1)\n"  /* reacquire argblock ptr (r5 is caller-save) */
+"        std  3,0(5)\n"    /* argblock[0] = r3 */
+
+/* pop off stack frame */
+"        ld      27,16(1)\n"        /* Fetch LR from frame */
+"        mtlr    27\n"              /* restore LR */
+"        ld      27,40(1)\n"        /* restore r27 from stack frame */
+"        addi    1,1,80\n"
 "        blr\n"
 "        .size do_syscall_WRK, .-do_syscall_WRK\n"
 );
@@ -615,6 +707,82 @@ asm(
 ".previous\n"
 );
 
+#elif defined(VGP_x86_freebsd)
+/* Incoming args (syscall number + up to 8 args) are on the stack.
+   FreeBSD has a syscall called 'syscall' that takes all args (including
+   the syscall number) off the stack.  Since we're called, the return
+   address is on the stack as expected, so we can just call syscall(2)
+   and it Just Works.  Error is when carry is set.
+*/
+extern ULong do_syscall_WRK (
+          UWord syscall_no, 
+          UWord a1, UWord a2, UWord a3,
+          UWord a4, UWord a5, UWord a6,
+          UWord a7, UWord a8, UInt *flags
+       );
+asm(
+".text\n"
+"do_syscall_WRK:\n"
+"      movl    $0,%eax\n"      /* syscall number = "syscall" (0) to avoid stack frobbing
+*/
+"      int     $0x80\n"
+"      jb      1f\n"
+"      ret\n"
+"1:    movl    40(%esp),%ecx\n"        /* store carry in *flags */
+"      movl    $1,(%ecx)\n"
+"      ret\n"
+".previous\n"
+);
+
+#elif defined(VGP_amd64_freebsd)
+extern UWord do_syscall_WRK (
+          UWord syscall_no,    /* %rdi */
+          UWord a1,            /* %rsi */
+          UWord a2,            /* %rdx */
+          UWord a3,            /* %rcx */
+          UWord a4,            /* %r8 */
+          UWord a5,            /* %r9 */
+          UWord a6,            /* 8(%rsp) */
+          UWord a7,            /* 16(%rsp) */
+          UWord a8,            /* 24(%rsp) */
+          UInt *flags,         /* 32(%rsp) */
+          UWord *rv2           /* 40(%rsp) */
+       );
+asm(
+".text\n"
+"do_syscall_WRK:\n"
+        /* Convert function calling convention --> syscall calling
+           convention */
+"      pushq   %rbp\n"
+"      movq    %rsp, %rbp\n"
+"      movq    %rdi, %rax\n"    /* syscall_no */
+"      movq    %rsi, %rdi\n"    /* a1 */
+"      movq    %rdx, %rsi\n"    /* a2 */
+"      movq    %rcx, %rdx\n"    /* a3 */
+"      movq    %r8,  %r10\n"    /* a4 */
+"      movq    %r9,  %r8\n"     /* a5 */
+"      movq    16(%rbp), %r9\n"  /* a6 last arg from stack, account for %rbp */
+"      movq    24(%rbp), %r11\n" /* a7 from stack */
+"      pushq  %r11\n"
+"      movq    32(%rbp), %r11\n" /* a8 from stack */
+"      pushq  %r11\n"
+"      subq    $8,%rsp\n"      /* fake return addr */
+"      syscall\n"
+"      jb      1f\n"
+"      movq    48(%rbp),%rsi\n"
+"      movq    %rdx, (%rsi)\n"
+"      movq    %rbp, %rsp\n"
+"      popq    %rbp\n"
+"      ret\n"
+"1:\n"
+"      movq    40(%rbp), %rsi\n"
+"      movl    $1,(%rsi)\n"
+"      movq    %rbp, %rsp\n"
+"      popq    %rbp\n"
+"      ret\n"
+".previous\n"
+);
+
 #elif defined(VGP_x86_darwin)
 
 /* Incoming args (syscall number + up to 8 args) come in on the stack
@@ -971,6 +1139,21 @@ SysRes VG_(do_syscall) ( UWord sysno, RegWord a1, RegWord a2, RegWord a3,
    UWord val = do_syscall_WRK(sysno,a1,a2,a3,a4,a5,a6);
    return VG_(mk_SysRes_amd64_linux)( val );
 
+#  elif defined(VGP_x86_freebsd)
+   ULong val;
+   UInt err = 0;
+   val = do_syscall_WRK(sysno, a1, a2, a3, a4, a5,
+                        a6, a7, a8, &err);
+   return VG_(mk_SysRes_x86_freebsd)( (UInt)val, (UInt)(val>>32), (err & 1) != 0 ? True : False);
+
+#  elif defined(VGP_amd64_freebsd)
+   UWord val;
+   UWord val2 = 0;
+   UInt err = 0;
+   val = do_syscall_WRK(sysno, a1, a2, a3, a4, a5,
+                        a6, a7, a8, &err, &val2);
+   return VG_(mk_SysRes_amd64_freebsd)( val, val2, (err & 1) != 0 ? True : False);
+
 #  elif defined(VGP_ppc32_linux)
    ULong ret     = do_syscall_WRK(sysno,a1,a2,a3,a4,a5,a6);
    UInt  val     = (UInt)(ret>>32);
@@ -978,7 +1161,11 @@ SysRes VG_(do_syscall) ( UWord sysno, RegWord a1, RegWord a2, RegWord a3,
    return VG_(mk_SysRes_ppc32_linux)( val, cr0so );
 
 #  elif defined(VGP_ppc64be_linux) || defined(VGP_ppc64le_linux)
-   ULong argblock[7];
+   ULong argblock[8];
+   /* PPC system calls have at most 6 arguments.  The Valgrind infrastructure
+      supports 8 system call arguments.  Argument 7 is used on PPC LE to pass
+      the flag indicating if the sc or scv instruction should be used for the
+      system call.  */
    argblock[0] = sysno;
    argblock[1] = a1;
    argblock[2] = a2;
@@ -986,8 +1173,9 @@ SysRes VG_(do_syscall) ( UWord sysno, RegWord a1, RegWord a2, RegWord a3,
    argblock[4] = a4;
    argblock[5] = a5;
    argblock[6] = a6;
+   argblock[7] = a7;
    do_syscall_WRK( &argblock[0] );
-   return VG_(mk_SysRes_ppc64_linux)( argblock[0], argblock[1] );
+   return VG_(mk_SysRes_ppc64_linux)( argblock[0], argblock[1], a7 );
 
 #  elif defined(VGP_arm_linux)
    UWord val = do_syscall_WRK(a1,a2,a3,a4,a5,a6,sysno);
@@ -1196,6 +1384,9 @@ const HChar* VG_(strerror) ( UWord errnum )
    case VKI_EOVERFLOW:   return "Value too large for defined data type";
 #     if defined(VKI_ERESTARTSYS)
       case VKI_ERESTARTSYS: return "ERESTARTSYS";
+#     endif
+#     if defined(VKI_ERESTART)
+      case VKI_ERESTART: return "ERESTART";
 #     endif
    default:              return "VG_(strerror): unknown error";
    }
