@@ -1009,11 +1009,15 @@ static SyscallTableEntry syscall_main_table[] = {
    LINXY(__NR_process_vm_readv,  sys_process_vm_readv), // 376
    LINX_(__NR_process_vm_writev, sys_process_vm_writev),// 377
 
+   LINX_(__NR_sched_setattr,     sys_sched_setattr),    // 380
+   LINXY(__NR_sched_getattr,     sys_sched_getattr),    // 381
    LINX_(__NR_renameat2,         sys_renameat2),        // 382
 
    LINXY(__NR_getrandom,         sys_getrandom),        // 384
    LINXY(__NR_memfd_create,      sys_memfd_create),     // 385
 
+   LINX_(__NR_execveat,          sys_execveat),         // 387
+
    LINX_(__NR_membarrier,        sys_membarrier),       // 389
    LINX_(__NR_copy_file_range,   sys_copy_file_range),  // 391
    LINXY(__NR_preadv2,           sys_preadv2),          // 392
@@ -1042,6 +1046,15 @@ static SyscallTableEntry syscall_main_table[] = {
    LINXY(__NR_futex_time64,      sys_futex_time64),     // 422
    LINXY(__NR_sched_rr_get_interval_time64,
          sys_sched_rr_get_interval_time64),             // 423
+
+   LINXY(__NR_io_uring_setup,    sys_io_uring_setup),    // 425
+   LINXY(__NR_io_uring_enter,    sys_io_uring_enter),    // 426
+   LINXY(__NR_io_uring_register, sys_io_uring_register), // 427
+
+   GENX_(__NR_clone3,            sys_ni_syscall),        // 435
+   LINXY(__NR_close_range,       sys_close_range),       // 436
+
+   LINX_(__NR_faccessat2,    sys_faccessat2),           // 439
 };
 
 
