@@ -825,14 +825,16 @@ static SyscallTableEntry syscall_table[] = {
    LINX_(__NR_kcmp, sys_kcmp),                                        // 343
 // ?????(__NR_finit_module, ),                                        // 344
 
-// ?????(__NR_sched_setattr, ),                                       // 345
-// ?????(__NR_sched_getattr, ),                                       // 346
+   LINX_(__NR_sched_setattr, sys_sched_setattr),                      // 345
+   LINXY(__NR_sched_getattr, sys_sched_getattr),                      // 346
    LINX_(__NR_renameat2, sys_renameat2),                              // 347
 // ?????(__NR_seccomp, ),                                             // 348
    LINXY(__NR_getrandom, sys_getrandom),                              // 349
 
    LINXY(__NR_memfd_create, sys_memfd_create),                        // 350
 
+   LINX_(__NR_execveat, sys_execveat),                                // 354
+
    LINX_(__NR_membarrier, sys_membarrier),                            // 356
    LINXY(__NR_recvmmsg, sys_recvmmsg),                                // 357
    LINXY(__NR_sendmmsg, sys_sendmmsg),                                // 358
@@ -857,6 +859,15 @@ static SyscallTableEntry syscall_table[] = {
    LINX_(__NR_pwritev2, sys_pwritev2),                                // 377
 
    LINXY(__NR_statx, sys_statx),                                      // 379
+
+   LINXY(__NR_io_uring_setup, sys_io_uring_setup),                    // 425
+   LINXY(__NR_io_uring_enter, sys_io_uring_enter),                    // 426
+   LINXY(__NR_io_uring_register, sys_io_uring_register),              // 427
+
+   GENX_(__NR_clone3, sys_ni_syscall),                                // 435
+   LINXY(__NR_close_range, sys_close_range),                          // 436
+
+   LINX_(__NR_faccessat2,  sys_faccessat2),                           // 439
 };
 
 SyscallTableEntry* ML_(get_linux_syscall_entry) ( UInt sysno )
