@@ -85,6 +85,13 @@ extern void VG_(do_atfork_pre)    ( ThreadId tid );
 extern void VG_(do_atfork_parent) ( ThreadId tid );
 extern void VG_(do_atfork_child)  ( ThreadId tid );
 
+#if defined(VGO_freebsd)
+// sysctl, modfind
+extern Int VG_(sysctlbyname)(const HChar *name, void *oldp, SizeT *oldlenp, const void *newp, SizeT newlen);
+extern Int VG_(getosreldate)(void);
+extern Bool VG_(is32on64)(void);
+#endif
+
 // icache invalidation
 extern void VG_(invalidate_icache) ( void *ptr, SizeT nbytes );
 
