@@ -1,7 +1,7 @@
 /*
   This file is part of drd, a thread error detector.
 
-  Copyright (C) 2006-2017 Bart Van Assche <bvanassche@acm.org>.
+  Copyright (C) 2006-2020 Bart Van Assche <bvanassche@acm.org>.
 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
@@ -629,6 +629,13 @@ static void drd_stop_using_mem_stack_signal(Addr a, SizeT len)
                       True);
 }
 
+static void drd_register_stack(Addr start, Addr end)
+{
+   DrdThreadId drd_tid = DRD_(thread_get_running_tid)();
+
+   DRD_(thread_register_stack)(drd_tid, start, end);
+}
+
 static
 void drd_pre_thread_create(const ThreadId creator, const ThreadId created)
 {
@@ -737,7 +744,7 @@ void drd__atfork_child(ThreadId tid)
 
 static void DRD_(post_clo_init)(void)
 {
-#if defined(VGO_linux) || defined(VGO_darwin) || defined(VGO_solaris)
+#if defined(VGO_linux) || defined(VGO_darwin) || defined(VGO_solaris) || defined(VGO_freebsd)
    /* fine */
 #else
    VG_(printf)("\nWARNING: DRD has not yet been tested on this operating system.\n\n");
@@ -822,7 +829,7 @@ void drd_pre_clo_init(void)
    VG_(details_name)            ("drd");
    VG_(details_version)         (NULL);
    VG_(details_description)     ("a thread error detector");
-   VG_(details_copyright_author)("Copyright (C) 2006-2017, and GNU GPL'd,"
+   VG_(details_copyright_author)("Copyright (C) 2006-2020, and GNU GPL'd,"
                                  " by Bart Van Assche.");
    VG_(details_bug_reports_to)  (VG_BUGS_TO);
 
@@ -852,6 +859,7 @@ void drd_pre_clo_init(void)
    VG_(track_die_mem_munmap)       (drd_stop_using_nonstack_mem);
    VG_(track_die_mem_stack)        (drd_stop_using_mem_stack);
    VG_(track_die_mem_stack_signal) (drd_stop_using_mem_stack_signal);
+   VG_(track_register_stack)       (drd_register_stack);
    VG_(track_pre_deliver_signal)   (drd_pre_deliver_signal);
    VG_(track_post_deliver_signal)  (drd_post_deliver_signal);
    VG_(track_start_client_code)    (drd_start_client_code);
