@@ -4,6 +4,9 @@
 #include <pthread.h>
 #include <semaphore.h>
 #include <unistd.h>
+#if defined(VGO_freebsd)
+# include <sys/fcntl.h>
+#endif
 /* This is really a test of semaphore handling
    (sem_{init,destroy,post,wait}).  Using semaphores a barrier
    function is created.  Helgrind-3.3 (p.k.a Thrcheck) does understand
@@ -104,7 +107,7 @@ gomp_barrier_wait (gomp_barrier_t *bar)
           my_sem_wait (bar->sem2); // 2 down
         }
       pthread_mutex_unlock (&bar->mutex1);
-      /* Resultats professionnels!  First we made this thread have an
+      /* "Resultats professionnels!"  First we made this thread have an
          obvious (Thrcheck-visible) dependency on all other threads
          calling gomp_barrier_wait.  Then, we released them all again,
          so they all have a (visible) dependency on this thread.
@@ -222,7 +225,7 @@ static sem_t* my_sem_init (char* identity, int pshared, unsigned count)
 {
    sem_t* s;
 
-#if defined(VGO_linux)
+#if defined(VGO_linux) || defined(VGO_solaris) || defined(VGO_freebsd)
    s = malloc(sizeof(*s));
    if (s) {
       if (sem_init(s, pshared, count) < 0) {
