@@ -7,8 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2010-2015 RT-RK
-      mips-valgrind@rt-rk.com
+   Copyright (C) 2010-2017 RT-RK
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -21,9 +20,7 @@
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-   02111-1307, USA.
+   along with this program; if not, see <http://www.gnu.org/licenses/>.
 
    The GNU General Public License is contained in the file COPYING.
 */
@@ -71,12 +68,12 @@ typedef unsigned long long __vki_u64;
 // From linux-2.6.35.5/include/asm-mips/page.h
 //----------------------------------------------------------------------
 
-/* PAGE_SHIFT determines the page size */
-#define VKI_PAGE_SHIFT          MIPS_PAGE_SHIFT
-#define VKI_PAGE_SIZE           (1UL << VKI_PAGE_SHIFT)
+/* MIPS32 uses runtime pagesize detection */
+extern UWord VKI_PAGE_SHIFT;
+extern UWord VKI_PAGE_SIZE;
 #define VKI_PAGE_MASK           (~(VKI_PAGE_SIZE-1))
-#define VKI_MAX_PAGE_SHIFT      VKI_PAGE_SHIFT
-#define VKI_MAX_PAGE_SIZE       VKI_PAGE_SIZE
+#define VKI_MAX_PAGE_SHIFT      16
+#define VKI_MAX_PAGE_SIZE       (1UL << VKI_MAX_PAGE_SHIFT)
 
 //----------------------------------------------------------------------
 // From linux-2.6.35.9/arch/mips/include/bits/shm.h
@@ -369,6 +366,7 @@ struct vki_f_owner_ex {
 #define VKI_RLIMIT_STACK	3   /* max stack size */
 #define VKI_RLIMIT_CORE		4   /* max core file size */
 #define VKI_RLIMIT_NOFILE	5   /* max number of open files */
+#define VKI_RLIM_INFINITY 0x7fffffffUL
 
 //----------------------------------------------------------------------
 // From linux-2.6.35.5/include/asm-mips/socket.h
@@ -384,9 +382,9 @@ struct vki_f_owner_ex {
 // From linux-2.6.35.5/include/asm-i386/sockios.h
 //----------------------------------------------------------------------
 
-#define VKI_SIOCSPGRP           0x8902
-#define VKI_SIOCGPGRP           0x8904
-#define VKI_SIOCATMARK          0x8905
+#define VKI_SIOCATMARK          _VKI_IOR('s', 7, int)
+#define VKI_SIOCSPGRP           _VKI_IOW('s', 8, vki_pid_t)
+#define VKI_SIOCGPGRP           _VKI_IOR('s', 9, vki_pid_t)
 #define VKI_SIOCGSTAMP          0x8906      /* Get stamp (timeval) */
 #define VKI_SIOCGSTAMPNS        0x8907      /* Get stamp (timespec) */
 
@@ -679,7 +677,7 @@ struct vki_ucontext {
 };
 
 // CAB: TODO
-typedef void vki_modify_ldt_t;
+typedef char vki_modify_ldt_t;
 
 //----------------------------------------------------------------------
 // From linux-2.6.35.5/include/asm-mips/ipcbuf.h
@@ -815,56 +813,60 @@ struct vki_pt_regs {
 #endif
 } __attribute__ ((aligned (8)));
 
-
-#define vki_user_regs_struct vki_pt_regs
-
-#define MIPS_lo  	lo
-#define MIPS_hi  	hi
-//#define MIPS_pc		regs[32]
-#define MIPS_r31		regs[31]
-#define MIPS_r30		regs[30]
-#define MIPS_r29		regs[29]
-#define MIPS_r28		regs[28]
-#define MIPS_r27		regs[27]
-#define MIPS_r26		regs[26]
-#define MIPS_r25		regs[25]
-#define MIPS_r24		regs[24]
-#define MIPS_r23		regs[23]
-#define MIPS_r22		regs[22]
-#define MIPS_r21		regs[21]
-#define MIPS_r20		regs[20]
-#define MIPS_r19		regs[19]
-#define MIPS_r18		regs[18]
-#define MIPS_r17		regs[17]
-#define MIPS_r16		regs[16]
-#define MIPS_r15		regs[15]
-#define MIPS_r14		regs[14]
-#define MIPS_r13		regs[13]
-#define MIPS_r12		regs[12]
-#define MIPS_r11		regs[11]
-#define MIPS_r10		regs[10]
-#define MIPS_r9		regs[9]
-#define MIPS_r8		regs[8]
-#define MIPS_r7		regs[7]
-#define MIPS_r6		regs[6]
-#define MIPS_r5		regs[5]
-#define MIPS_r4		regs[4]
-#define MIPS_r3		regs[3]
-#define MIPS_r2		regs[2]
-#define MIPS_r1		regs[1]
-#define MIPS_r0		regs[0]
-
 #define VKI_PTRACE_GETREGS            12
 #define VKI_PTRACE_SETREGS            13
 #define VKI_PTRACE_GETFPREGS          14
 #define VKI_PTRACE_SETFPREGS          15
 //----------------------------------------------------------------------
-// From linux-2.6.35.5/include/asm-mips/elf.h
+// From linux-4.5/arch/mips/include/uapi/asm/reg.h
+//----------------------------------------------------------------------
+#define VKI_MIPS32_EF_R0               6
+#define VKI_MIPS32_EF_R1               7
+#define VKI_MIPS32_EF_R2               8
+#define VKI_MIPS32_EF_R3               9
+#define VKI_MIPS32_EF_R4              10
+#define VKI_MIPS32_EF_R5              11
+#define VKI_MIPS32_EF_R6              12
+#define VKI_MIPS32_EF_R7              13
+#define VKI_MIPS32_EF_R8              14
+#define VKI_MIPS32_EF_R9              15
+#define VKI_MIPS32_EF_R10             16
+#define VKI_MIPS32_EF_R11             17
+#define VKI_MIPS32_EF_R12             18
+#define VKI_MIPS32_EF_R13             19
+#define VKI_MIPS32_EF_R14             20
+#define VKI_MIPS32_EF_R15             21
+#define VKI_MIPS32_EF_R16             22
+#define VKI_MIPS32_EF_R17             23
+#define VKI_MIPS32_EF_R18             24
+#define VKI_MIPS32_EF_R19             25
+#define VKI_MIPS32_EF_R20             26
+#define VKI_MIPS32_EF_R21             27
+#define VKI_MIPS32_EF_R22             28
+#define VKI_MIPS32_EF_R23             29
+#define VKI_MIPS32_EF_R24             30
+#define VKI_MIPS32_EF_R25             31
+#define VKI_MIPS32_EF_R26             32
+#define VKI_MIPS32_EF_R27             33
+#define VKI_MIPS32_EF_R28             34
+#define VKI_MIPS32_EF_R29             35
+#define VKI_MIPS32_EF_R30             36
+#define VKI_MIPS32_EF_R31             37
+#define VKI_MIPS32_EF_LO              38
+#define VKI_MIPS32_EF_HI              39
+#define VKI_MIPS32_EF_CP0_EPC         40
+#define VKI_MIPS32_EF_CP0_BADVADDR    41
+#define VKI_MIPS32_EF_CP0_STATUS      42
+#define VKI_MIPS32_EF_CP0_CAUSE       43
+#define VKI_MIPS32_EF_UNUSED0         44
+
+//----------------------------------------------------------------------
+// From linux-4.5/arch/mips/include/asm/elf.h
 //----------------------------------------------------------------------
 typedef unsigned long vki_elf_greg_t;
 
-#define VKI_ELF_NGREG (sizeof (struct vki_user_regs_struct) / sizeof(vki_elf_greg_t))
-#define VKI_ELF_NFPREG			33	/* includes fpscr */
+#define VKI_ELF_NGREG      45
+#define VKI_ELF_NFPREG     33 /* includes fpscr */
 
 typedef vki_elf_greg_t vki_elf_gregset_t[VKI_ELF_NGREG];
 
@@ -981,6 +983,14 @@ enum vki_sock_type {
 };
 #define ARCH_HAS_SOCKET_TYPES 1
 
+//----------------------------------------------------------------------
+// From linux-3.7.0/arch/mips/include/uapi/asm/errno.h
+//----------------------------------------------------------------------
+
+#define VKI_ELIBBAD      84  /* Accessing a corrupted shared library */
+#define VKI_EOPNOTSUPP   122 /* Operation not supported on transport
+                                 endpoint */
+
 //----------------------------------------------------------------------
 // From linux-3.13.0/include/asm/errno.h
 //----------------------------------------------------------------------
@@ -988,6 +998,65 @@ enum vki_sock_type {
 #define	VKI_ENOSYS       89  /* Function not implemented */
 #define	VKI_EOVERFLOW    79  /* Value too large for defined data type */
 
+//----------------------------------------------------------------------
+// From linux-3.14.0/arch/mips/include/asm/elf.h
+//----------------------------------------------------------------------
+
+#define VKI_EF_MIPS_FP64    0x00000200
+
+//----------------------------------------------------------------------
+// From linux-4.1.0/arch/mips/include/asm/elf.h
+//----------------------------------------------------------------------
+
+#define VKI_MIPS_ABI_FP_UNKNOWN (-1)
+#define VKI_MIPS_ABI_FP_ANY       0  /* FP ABI doesn't matter */
+#define VKI_MIPS_ABI_FP_DOUBLE    1  /* -mdouble-float */
+#define VKI_MIPS_ABI_FP_SINGLE    2  /* -msingle-float */
+#define VKI_MIPS_ABI_FP_SOFT      3  /* -msoft-float */
+#define VKI_MIPS_ABI_FP_OLD_64    4  /* -mips32r2 -mfp64 */
+#define VKI_MIPS_ABI_FP_XX        5  /* -mfpxx */
+#define VKI_MIPS_ABI_FP_64        6  /* -mips32r2 -mfp64 */
+#define VKI_MIPS_ABI_FP_64A       7  /* -mips32r2 -mfp64 -mno-odd-spreg */
+
+struct vki_arch_elf_state {
+   int fp_abi;
+   int interp_fp_abi;
+   int overall_fp_mode;
+};
+
+#define VKI_INIT_ARCH_ELF_STATE {             \
+   .fp_abi = VKI_MIPS_ABI_FP_UNKNOWN,         \
+   .interp_fp_abi = VKI_MIPS_ABI_FP_UNKNOWN,  \
+   .overall_fp_mode = -1,                     \
+}
+
+struct vki_mips_elf_abiflags_v0 {
+   vki_u16 version;     /* Version of flags structure */
+   vki_u8  isa_level;   /* The level of the ISA: 1-5, 32, 64 */
+   vki_u8  isa_rev;     /* The revision of ISA: 0 for MIPS V and below,
+                           1-n otherwise */
+   vki_u8  gpr_size;    /* The size of general purpose registers */
+   vki_u8  cpr1_size;   /* The size of co-processor 1 registers */
+   vki_u8  cpr2_size;   /* The size of co-processor 2 registers */
+   vki_u8  fp_abi;      /* The floating-point ABI */
+   vki_u32 isa_ext;     /* Mask of processor-specific extensions */
+   vki_u32 ases;        /* Mask of ASEs used */
+   vki_u32 flags1;      /* Mask of general flags */
+   vki_u32 flags2;
+};
+
+#define VKI_PT_MIPS_ABIFLAGS   0x70000003
+
+//----------------------------------------------------------------------
+// From linux-4.1.0/arch/mips/kernel/elf.c
+//----------------------------------------------------------------------
+
+enum {
+   VKI_FP_FRE,
+   VKI_FP_FR0,
+   VKI_FP_FR1,
+};
+
 #endif // __VKI_MIPS32_LINUX_H
 
 /*--------------------------------------------------------------------*/
