@@ -13,22 +13,22 @@
  General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- 02111-1307, USA.
+ along with this program; if not, see <http://www.gnu.org/licenses/>.
 
  The GNU General Public License is contained in the file COPYING.
  */
 
-#ifdef HAS_VSX
-
 #include <stdio.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <malloc.h>
-#include <altivec.h>
 #include <math.h>
+#include <unistd.h>    // getopt
+
+#ifdef HAS_VSX
+
+#include <altivec.h>
 
 #ifndef __powerpc64__
 typedef uint32_t HWord_t;
@@ -36,6 +36,12 @@ typedef uint32_t HWord_t;
 typedef uint64_t HWord_t;
 #endif /* __powerpc64__ */
 
+#ifdef VGP_ppc64le_linux
+#define isLE 1
+#else
+#define isLE 0
+#endif
+
 typedef unsigned char Bool;
 #define True 1
 #define False 0
@@ -85,6 +91,9 @@ static volatile unsigned int div_flags, div_xer;
 typedef void (*test_func_t)(void);
 typedef struct test_table test_table_t;
 
+/* Defines for the instructiion groups, use bit field to identify */
+#define SCALAR_DIV_INST    0x0001
+#define OTHER_INST  0x0002
 
 /* These functions below that construct a table of floating point
  * values were lifted from none/tests/ppc32/jm-insns.c.
@@ -107,6 +116,14 @@ static inline void register_farg (void *farg,
               s, _exp, mant, *(uint64_t *)farg, *(double *)farg);
 }
 
+static inline void register_sp_farg (void *farg,
+                                     int s, uint16_t _exp, uint32_t mant)
+{
+   uint32_t tmp;
+   tmp = ((uint32_t)s << 31) | ((uint32_t)_exp << 23) | mant;
+   *(uint32_t *)farg = tmp;
+}
+
 
 typedef struct fp_test_args {
    int fra_idx;
@@ -217,6 +234,7 @@ static void build_special_fargs_table(void)
 */
 
    uint64_t mant;
+   uint32_t mant_sp;
    uint16_t _exp;
    int s;
    int j, i = 0;
@@ -287,28 +305,42 @@ static void build_special_fargs_table(void)
    mant = 0x0000000000000ULL;
    register_farg(&spec_fargs[i++], s, _exp, mant);
 
-   /* +QNaN     : 0 0x7FF 0x7FFFFFFFFFFFF */
+   /*
+    * This comment applies to values #9 and #10 below:
+    * When src is a SNaN, it's converted to a QNaN first before rounding to single-precision,
+    * so we can't just copy the double-precision value to the corresponding slot in the
+    * single-precision array (i.e., in the loop at the end of this function).  Instead, we
+    * have to manually set the bits using register_sp_farg().
+    */
+
+   /* +SNaN     : 0 0x7FF 0x7FFFFFFFFFFFF */
    // #9
    s = 0;
    _exp = 0x7FF;
    mant = 0x7FFFFFFFFFFFFULL;
    register_farg(&spec_fargs[i++], s, _exp, mant);
+   _exp = 0xff;
+   mant_sp = 0x3FFFFF;
+   register_sp_farg(&spec_sp_fargs[i-1], s, _exp, mant_sp);
 
-   /* -QNaN     : 1 0x7FF 0x7FFFFFFFFFFFF */
+   /* -SNaN     : 1 0x7FF 0x7FFFFFFFFFFFF */
    // #10
    s = 1;
    _exp = 0x7FF;
    mant = 0x7FFFFFFFFFFFFULL;
    register_farg(&spec_fargs[i++], s, _exp, mant);
+   _exp = 0xff;
+   mant_sp = 0x3FFFFF;
+   register_sp_farg(&spec_sp_fargs[i-1], s, _exp, mant_sp);
 
-   /* +SNaN     : 0 0x7FF 0x8000000000000 */
+   /* +QNaN     : 0 0x7FF 0x8000000000000 */
    // #11
    s = 0;
    _exp = 0x7FF;
    mant = 0x8000000000000ULL;
    register_farg(&spec_fargs[i++], s, _exp, mant);
 
-   /* -SNaN     : 1 0x7FF 0x8000000000000 */
+   /* -QNaN     : 1 0x7FF 0x8000000000000 */
    // #12
    s = 1;
    _exp = 0x7FF;
@@ -362,7 +394,8 @@ static void build_special_fargs_table(void)
 
    nb_special_fargs = i;
    for (j = 0; j < i; j++) {
-      spec_sp_fargs[j] = spec_fargs[j];
+      if (!(j == 9 || j == 10))
+         spec_sp_fargs[j] = spec_fargs[j];
    }
 }
 
@@ -371,6 +404,7 @@ struct test_table
 {
    test_func_t test_category;
    char * name;
+   unsigned int test_group;
 };
 
 /*  Type of input for floating point operations.*/
@@ -729,6 +763,11 @@ static void test_xvnegdp(void)
    __asm__ __volatile__ ("xvnegdp   %x0, %x1" : "=wa" (vec_out): "wa" (vec_inB));
 }
 
+static void test_xvnegsp(void)
+{
+   __asm__ __volatile__ ("xvnegsp   %x0, %x1" : "=wa" (vec_out): "wa" (vec_inB));
+}
+
 static void test_xvabssp(void)
 {
    __asm__ __volatile__ ("xvabssp   %x0, %x1" : "=wa" (vec_out): "wa" (vec_inB));
@@ -814,6 +853,7 @@ vsx_one_fp_arg_tests[] = {
                                 { &test_xsrdpi, "xsrdpi", NULL, 20, DOUBLE_TEST, VX_CONV_TO_DOUBLE, "round"},
                                 { &test_xvabsdp, "xvabsdp", NULL, 20, DOUBLE_TEST, VX_DEFAULT, "abs"},
                                 { &test_xvnabsdp, "xvnabsdp", NULL, 20, DOUBLE_TEST, VX_DEFAULT, "nabs"},
+                                { &test_xvnegsp, "xvnegsp", NULL, 20, SINGLE_TEST, VX_DEFAULT, "neg"},
                                 { &test_xvnegdp, "xvnegdp", NULL, 20, DOUBLE_TEST, VX_DEFAULT, "neg"},
                                 { &test_xvabssp, "xvabssp", NULL, 20, SINGLE_TEST, VX_DEFAULT, "abs"},
                                 { &test_xvnabssp, "xvnabssp", NULL, 20, SINGLE_TEST, VX_DEFAULT, "nabs"},
@@ -1092,7 +1132,7 @@ static void test_vsx_one_fp_arg(void)
 
       for (i = 0; i < test_group.num_tests; i+=stride) {
          unsigned int * pv;
-         void * inB;
+         void * inB, * vecB_void_ptr = (void *)&vec_inB;
 
          pv = (unsigned int *)&vec_out;
          // clear vec_out
@@ -1105,11 +1145,15 @@ static void test_vsx_one_fp_arg(void)
             for (j = 0; j < loops; j++) {
                inB = (void *)&spec_fargs[i + j];
                // copy double precision FP into vector element i
-               memcpy(((void *)&vec_inB) + (j * 8), inB, 8);
+               if (isLE && is_scalar)
+                  vecB_void_ptr += 8;
+               memcpy(vecB_void_ptr + (j * 8), inB, 8);
             }
             // execute test insn
             (*func)();
             dst_dp = (unsigned long long *) &vec_out;
+            if (isLE && is_scalar)
+               dst_dp++;
             printf("#%d: %s ", i/stride, test_group.name);
             for (j = 0; j < loops; j++) {
                if (j)
@@ -1117,7 +1161,7 @@ static void test_vsx_one_fp_arg(void)
                frB_dp = (unsigned long long *)&spec_fargs[i + j];
                printf("%s(%016llx)", test_group.op, *frB_dp);
                if (estimate) {
-                  Bool res = check_estimate(DOUBLE_TEST, is_sqrt, i + j, j);
+                  Bool res = check_estimate(DOUBLE_TEST, is_sqrt, i + j, (isLE && is_scalar) ? 1: j);
                   printf(" ==> %s)", res ? "PASS" : "FAIL");
                   /* For debugging . . .
                    printf(" ==> %s (res=%016llx)", res ? "PASS" : "FAIL", dst_dp[j]);
@@ -1138,29 +1182,36 @@ static void test_vsx_one_fp_arg(void)
             }
             printf("\n");
          } else {
-            int j, skip_slot;
+            int j;
             unsigned int * frB_sp, * dst_sp = NULL;
             unsigned long long * dst_dp = NULL;
-            if (sparse_sp) {
-               skip_slot = 1;
+            if (sparse_sp)
                loops = 2;
-            } else {
-               skip_slot = 0;
-            }
             for (j = 0; j < loops; j++) {
                inB = (void *)&spec_sp_fargs[i + j];
                // copy single precision FP into vector element i
-               if (skip_slot && j > 0)
-                  memcpy(((void *)&vec_inB) + ((j + j) * 4), inB, 4);
-               else
-                  memcpy(((void *)&vec_inB) + (j * 4), inB, 4);
+               if (sparse_sp) {
+                  if (isLE)
+                     memcpy(vecB_void_ptr + ((2 * j * 4) + 4), inB, 4);
+                  else
+                     memcpy(vecB_void_ptr + ((2 * j * 4) ), inB, 4);
+               } else {
+                  if (isLE && is_scalar)
+                     vecB_void_ptr += 12;
+                  memcpy(vecB_void_ptr + (j * 4), inB, 4);
+               }
             }
             // execute test insn
             (*func)();
-            if (test_group.type == VX_CONV_TO_DOUBLE)
+            if (test_group.type == VX_CONV_TO_DOUBLE) {
                dst_dp = (unsigned long long *) &vec_out;
-            else
+               if (isLE && is_scalar)
+                  dst_dp++;
+            } else {
                dst_sp = (unsigned int *) &vec_out;
+               if (isLE && is_scalar)
+                  dst_sp += 3;
+            }
             // print result
             printf("#%d: %s ", i/stride, test_group.name);
             for (j = 0; j < loops; j++) {
@@ -1169,7 +1220,7 @@ static void test_vsx_one_fp_arg(void)
                frB_sp = (unsigned int *)&spec_sp_fargs[i + j];
                printf("%s(%08x)", test_group.op, *frB_sp);
                if (estimate) {
-                  Bool res = check_estimate(SINGLE_TEST, is_sqrt, i + j, j);
+                  Bool res = check_estimate(SINGLE_TEST, is_sqrt, i + j, (isLE && is_scalar) ? 3 : j);
                   printf(" ==> %s)", res ? "PASS" : "FAIL");
                } else {
                   if (test_group.type == VX_CONV_TO_DOUBLE)
@@ -1251,23 +1302,24 @@ static void test_int_to_fp_convert(void)
             }
             printf("\n");
          } else {
-            int j, skip_slot;
+            int j;
             unsigned int * dst_sp = NULL;
             unsigned int * targs = test_group.targs;
             unsigned long long * dst_dp = NULL;
-            if (sparse_sp) {
-               skip_slot = 1;
+            void * vecB_void_ptr = (void *)&vec_inB;
+            if (sparse_sp)
                loops = 2;
-            } else {
-               skip_slot = 0;
-            }
             for (j = 0; j < loops; j++) {
                inB = (void *)&targs[i + j];
                // copy single word into vector element i
-               if (skip_slot && j > 0)
-                  memcpy(((void *)&vec_inB) + ((j + j) * 4), inB, 4);
-               else
-                  memcpy(((void *)&vec_inB) + (j * 4), inB, 4);
+               if (sparse_sp) {
+                  if (isLE)
+                     memcpy(vecB_void_ptr + ((2 * j * 4) + 4), inB, 4);
+                  else
+                     memcpy(vecB_void_ptr + ((2 * j * 4) ), inB, 4);
+               } else {
+                  memcpy(vecB_void_ptr + (j * 4), inB, 4);
+               }
             }
             // execute test insn
             (*func)();
@@ -1303,12 +1355,12 @@ signed long long div_dw_tdata[13][2] = {
                                        { 4, 4 },
                                        { 4, -5 },
                                        { 3, 8 },
-                                       { 0x8000000000000000, 0xa },
+                                       { 0x8000000000000000ULL, 0xa },
                                        { 0x50c, -1 },
                                        { 0x50c, -4096 },
                                        { 0x1234fedc, 0x8000a873 },
-                                       { 0xabcd87651234fedc, 0xa123b893 },
-                                       { 0x123456789abdc, 0 },
+                                       { 0xabcd87651234fedcULL, 0xa123b893 },
+                                       { 0x123456789abdcULL, 0 },
                                        { 0, 2 },
                                        { 0x77, 0xa3499 }
 };
@@ -1417,7 +1469,7 @@ static void test_vx_tdivORtsqrt(void)
 
       for (i = 0; i < test_group.num_tests; i+=stride) {
          unsigned int * pv;
-         void * inB;
+         void * inB, * vecB_void_ptr = (void *)&vec_inB;
 
          pv = (unsigned int *)&vec_out;
          // clear vec_out
@@ -1433,7 +1485,9 @@ static void test_vx_tdivORtsqrt(void)
                for (j = 0; j < loops; j++) {
                   inB = (void *)&spec_fargs[i + j];
                   // copy double precision FP into vector element i
-                  memcpy(((void *)&vec_inB) + (j * 8), inB, 8);
+                  if (isLE && is_scalar)
+                     vecB_void_ptr += 8;
+                  memcpy(vecB_void_ptr + (j * 8), inB, 8);
                }
             }
             // execute test insn
@@ -1542,33 +1596,81 @@ static test_table_t
 {
 
                     { &test_vsx_one_fp_arg,
-                      "Test VSX vector and scalar single argument instructions"} ,
+                      "Test VSX vector and scalar single argument instructions", OTHER_INST } ,
                     { &test_int_to_fp_convert,
-                      "Test VSX vector integer to float conversion instructions" },
+                      "Test VSX vector integer to float conversion instructions", OTHER_INST },
                     { &test_div_extensions,
-                       "Test div extensions" },
+		      "Test div extensions", SCALAR_DIV_INST },
                     { &test_ftsqrt,
-                       "Test ftsqrt instruction" },
+		      "Test ftsqrt instruction", OTHER_INST },
                     { &test_vx_tdivORtsqrt,
-                       "Test vector and scalar tdiv and tsqrt instructions" },
+		      "Test vector and scalar tdiv and tsqrt instructions", OTHER_INST },
                     { &test_popcntw,
-                       "Test popcntw instruction" },
+		      "Test popcntw instruction", OTHER_INST },
                     { NULL, NULL }
 };
 #endif // HAS_VSX
 
-int main(int argc, char *argv[])
+static void usage (void)
+{
+  fprintf(stderr,
+	  "Usage: test_isa_3_0 [OPTIONS]\n"
+	  "\t-d: test scalar division instructions (default)\n"
+	  "\t-o: test non scalar division instructions (default)\n"
+	  "\t-A: test all instructions (default)\n"
+	  "\t-h: display this help and exit\n"
+	  );
+}
+
+int main(int argc, char **argv)
 {
 #ifdef HAS_VSX
 
    test_table_t aTest;
    test_func_t func;
+   int c;
    int i = 0;
+   unsigned int test_run_mask = 0;
+
+   /* NOTE, ISA 3.0 introduces the OV32 and CA32 bits in the FPSCR. These
+    * bits are set on various arithimetic instructions.  This means this
+    * test generates different FPSCR output for pre ISA 3.0 versus ISA 3.0
+    * hardware.  The tests have been grouped so that the tests that generate
+    * different results are in one test and the rest are in a different test.
+    * this minimizes the size of the result expect files for the two cases.
+    */
+
+   while ((c = getopt(argc, argv, "doAh")) != -1) {
+      switch (c) {
+      case 'd':
+	test_run_mask |= SCALAR_DIV_INST;
+         break;
+      case 'o':
+	test_run_mask |= OTHER_INST;
+         break;
+      case 'A':
+	test_run_mask = 0xFFFF;
+         break;
+      case 'h':
+         usage();
+         return 0;
+
+      default:
+         usage();
+         fprintf(stderr, "Unknown argument: '%c'\n", c);
+         return 1;
+      }
+   }
 
    while ((func = all_tests[i].test_category)) {
       aTest = all_tests[i];
-      printf( "%s\n", aTest.name );
-      (*func)();
+
+      if(test_run_mask & aTest.test_group) {
+	/* Test group  specified on command line */
+
+	printf( "%s\n", aTest.name );
+	(*func)();
+      }
       i++;
    }
    if (spec_fargs)
